#' @name mvqqplot
#' @rdname mvqqplot
#' @title qqplot for two multivariate datasets
#' 
#' @description Take two matrices and display a set of qqplots for
#' specified functions of their columns, as a quick way of assessing
#' whether they represent samples from similar distributions.  The
#' functions are slices through the p-dimensional Rosenbrock banana
#' function, where p is the (common) number of columns in the two
#' matrices.  The default is to look at the p axial slices, but other
#' margins can be specified: see the Example for \code{\link{slice}}.
#'
#' @param X,Y two matrices with the same number of columns (\code{p})
#' @param subsets list of margins to plot
#' @param N number of points to show (see Details)
#' @param trim drop the \code{trim} largest points (proportion), which tend to be very variable in qqplots.
#'
#' @return Invoked for its side-effect, of creating a set of qqplots
#' of the specified functions.
#'
#' @details If the number of rows of the two matrices differ, then the larger matrix is reduced by resampling its rows.  Both matrices are reduced by resampling to \code{N} rows if the smaller matrix has at least \code{N} rows.
#'
#' @seealso \code{\link{qqplot}} for one-dimensional comparisons.
#'
#' @author Jonathan Rougier <j.c.rougier@@bristol.ac.uk>
#' 
NULL

#' @rdname mvqqplot
#' @export
#'
#' @examples
#' ## Here are some samples from the same distribution
#'
#' A <- matrix(c(1, -1, 1, 2), ncol = 2)
#' X <- matrix(70 + rnorm(200), ncol = 2) %*% A
#' Y <- matrix(70 + rnorm(250), ncol = 2) %*% A
#' mvqqplot(X, Y) # do this a few times to see sampling variation
#' mvqqplot(X, Y, list(1, 2, c(1, 2))) # see other margins
#'
#' ## And here are some from different distributions
#'
#' Y <- matrix(65 + rt(250, df = 3), ncol = 2) %*% A
#' mvqqplot(X, Y) # Yikes!
#' mvqqplot(X, Y, list(1, 2, c(1, 2)))

mvqqplot <- function(X, Y, subsets = as.list(1L:p), N = 200, trim = 0.1) {

  X <- as.matrix(X)
  Y <- as.matrix(Y)
  stopifnot(ncol(X) == ncol(Y))
  p <- ncol(X)

  ## standardise X and Y to a common scale and then map

  tmp <- scale(rbind(X, Y))
  stopifnot(unlist(subsets) %in% 1L:p)
  eps <- 1e-6
  if (p > 1) {
    tmp <- sapply(subsets, function(ss) {
      if (length(ss) == 1L)
        tmp[, ss]
      else { # no map for univariate margins
        tmp[, -ss] <- 0 # no effect when ss = 1L:d
        log(eps + apply(tmp, 1, banana)) # banana(1, 1, ..., 1) = 0
      }
    })
  }
  X <- tmp[1L:nrow(X), , drop=FALSE]
  Y <- tmp[-(1L:nrow(X)), , drop=FALSE]

  ## resample to match the number of rows

  nx <- nrow(X); ny <- nrow(Y); nz <- min(c(nx, ny, N))
  if (nx > nz)
    X <- X[sample.int(n = nx, size = nz), , drop=FALSE]
  if (ny > nz)
    Y <- Y[sample.int(n = ny, size = nz), , drop=FALSE]

  ## set up the plotting window

  k <- length(subsets)
  if (k > 1) {
    mfrow <- c(floor(sqrt(k)), ceiling(sqrt(k)))
    if (prod(mfrow) < k)
      mfrow[1] <- mfrow[1] + 1
  } else mfrow = c(1, 1)
  op <- par(mfrow = mfrow, mar = c(2, 2, 2, 1), mgp = c(0.5, 0, 0), cex.main = 1)
  on.exit(par(op))

  ## do the qqplots for each column

  if (length(trim) == 1)
    trim <- c(trim/2, 1 - trim/2)

  for (i in 1:k) {
    x <- sort(X[, i])
    y <- sort(Y[, i])
    lim <- quantile(c(x, y), trim[1L:2], names = FALSE)
    plot.new()
    plot.window(lim, lim, asp = 1)
    lines(lim, lim, col = "blue", lty = 2)
    rect(lim[1], lim[1], lim[2], lim[2])
    keep <- lim[1] <= pmin(x, y) & pmax(x, y) <= lim[2]
    points(cbind(x, y)[keep, , drop=FALSE], pch = 20)
    main <- paste0("Subset [", paste(subsets[[i]], collapse = ", "), "]")
    title(main = main, xlab = "Values for x", ylab = "Values for y")
  }

  invisible(NULL)
}

#' @rdname mvqqplot
#' @param x arguments to Rosenbrock banana function
#' @param theta coefficient in Rosenbrock banana function
#' @details The Rosenbrock banana function with parameter \eqn{\theta} is defined as
#' \deqn{\sum_{i=1}^{p-1} [ (1 - x_i)^2 + \theta \times (x_{i+1} - x_i^2)^2 ] }
#' See \url{http://en.wikipedia.org/wiki/Rosenbrock_function} for more details.  The log of the banana function is simply used here as an interesting non-linear function of several variables.
#' @export

banana <- function(x, theta = 100) {
  p <- length(x)
  if (p == 1)
    stop("Expecting at least two components in \'x\'")
  i <- 1:(p-1)
  sum((1 - x[i])^2 + theta * (x[i+1] - x[i]^2)^2)
}
